@@ -1,4 +1,7 @@
-use crossterm::{cursor::MoveTo, event, terminal, ExecutableCommand};
+use crossterm::{
+    cursor::{MoveLeft, MoveTo},
+    event, terminal, ExecutableCommand,
+};
 use dirs::home_dir;
 use serde::{Deserialize, Serialize};
 use text_io::read;
@@ -6,7 +9,7 @@ use text_io::read;
 use egg_mode::{
     auth::{access_token, authorize_url, request_token},
     error::{Error::TwitterError, TwitterErrors},
-    tweet::{like, mentions_timeline, DraftTweet, Tweet},
+    tweet::{like, mentions_timeline, show, DraftTweet, Tweet},
     KeyPair, Token,
 };
 
@@ -23,14 +26,14 @@ use tui::{
     layout::{Constraint, Layout},
     style,
     text::{Span, Spans},
-    widgets::{Block, Borders, Paragraph},
+    widgets::{Block, Borders, Paragraph, Wrap},
     Terminal,
 };
 
 #[derive(Clone, Deserialize, Serialize)]
 struct Keys {
-    key: String,
-    secret: String,
+    key: Option<String>,
+    secret: Option<String>,
     access_token: Option<String>,
     access_token_secret: Option<String>,
 }
@@ -38,13 +41,18 @@ struct Keys {
 type Term = Terminal<CrosstermBackend<Stdout>>;
 type TwitterResult = Result<(), TwitterErrors>;
 
-const MENU_TEXT: &str = "(l)ike | (r)eply | (n)ext | e(x)it";
+const CFG_PATH: &str = ".config/qt/keys.toml";
+const MENU_TEXT: &str = "(l)ike | (r)eply | (t)weet | (n)ext | (p)revious | (P)arent | e(x)it";
 
 async fn authenticate() -> Token {
-    let key_file = home_dir().unwrap().join(".config/qt/keys.toml");
+    let key_file = home_dir().unwrap().join(CFG_PATH);
     let key_map: Keys = toml::from_str(&fs::read_to_string(&key_file).unwrap()).unwrap();
+    if !(key_map.key.is_some() && key_map.secret.is_some()) {
+        println!("Please put your consumer key and secret in ~/.config/qt/keys.toml");
+        exit(1);
+    };
     if key_map.access_token.is_some() {
-        let consumer_pair = KeyPair::new(key_map.key, key_map.secret);
+        let consumer_pair = KeyPair::new(key_map.key.unwrap(), key_map.secret.unwrap());
         let access_token_pair = KeyPair::new(
             key_map.access_token.unwrap(),
             key_map.access_token_secret.unwrap(),
@@ -55,8 +63,9 @@ async fn authenticate() -> Token {
             access: access_token_pair,
         };
     }
+
     let mut config_file = key_map.clone();
-    let consumer_pair = KeyPair::new(key_map.key, key_map.secret);
+    let consumer_pair = KeyPair::new(key_map.key.unwrap(), key_map.secret.unwrap());
 
     let auth_token = request_token(&consumer_pair, "oob").await.unwrap();
     let auth_url = authorize_url(&auth_token);
@@ -67,9 +76,10 @@ async fn authenticate() -> Token {
     stdout().flush().unwrap();
     let verifier: String = read!();
     println!();
-    let (token, _, _) = access_token(consumer_pair, &auth_token, verifier)
+    let token = access_token(consumer_pair, &auth_token, verifier)
         .await
-        .unwrap();
+        .unwrap()
+        .0;
 
     match token {
         Token::Access {
@@ -88,11 +98,11 @@ async fn authenticate() -> Token {
 }
 
 async fn get_mentions(token: &Token) -> Vec<Tweet> {
-    mentions_timeline(&token).start().await.unwrap().1.to_vec()
+    mentions_timeline(token).start().await.unwrap().1.to_vec()
 }
 
 async fn send_like(tweet: &Tweet, token: &Token) -> TwitterResult {
-    match like(tweet.id, &token).await {
+    match like(tweet.id, token).await {
         Ok(_) => Ok(()),
         Err(TwitterError(_, e)) => Err(e),
         Err(_) => panic!(),
@@ -108,7 +118,7 @@ async fn send_reply(tweet: &Tweet, reply_text: String, token: &Token) -> Twitter
     );
     let result = DraftTweet::new(reply_text)
         .in_reply_to(tweet.id)
-        .send(&token)
+        .send(token)
         .await;
     match result {
         Ok(_) => Ok(()),
@@ -117,19 +127,31 @@ async fn send_reply(tweet: &Tweet, reply_text: String, token: &Token) -> Twitter
     }
 }
 
-fn err_view(errs: TwitterErrors) -> String {
-    errs.errors[0].message.clone()
+async fn send_tweet(text: String, token: &Token) -> TwitterResult {
+    let result = DraftTweet::new(text).send(token).await;
+    match result {
+        Ok(_) => Ok(()),
+        Err(TwitterError(_, e)) => Err(e),
+        Err(_) => panic!(),
+    }
 }
 
-fn tweet_view<'a>(tweet: &'a Tweet) -> Vec<Spans<'a>> {
+fn result_view(result: TwitterResult) -> String {
+    match result {
+        Ok(()) => "Done!".to_string(),
+        Err(errs) => errs.errors[0].to_string(),
+    }
+}
+
+fn tweet_view(tweet: &Tweet) -> Vec<Spans> {
     let screen_name = &tweet.user.as_ref().unwrap().screen_name;
-    let screen_name_msg = format!("@{} says:\n", screen_name);
+    let screen_name_msg = format!("@{} says:", screen_name);
     let in_response_msg = match &tweet.in_reply_to_screen_name {
         None => "".to_string(),
-        Some(name) => format!("in response to @{}\n", name),
+        Some(name) => format!("in response to @{}", name),
     };
 
-    vec![
+    let mut spans = vec![
         Spans::from(Span::styled(
             screen_name_msg,
             style::Style::default().add_modifier(style::Modifier::BOLD),
@@ -139,17 +161,19 @@ fn tweet_view<'a>(tweet: &'a Tweet) -> Vec<Spans<'a>> {
             style::Style::default().add_modifier(style::Modifier::ITALIC),
         )),
         Spans::from(Span::raw(&tweet.text)),
-    ]
+    ];
+    if tweet.in_reply_to_screen_name.is_none() {
+        spans.remove(1);
+    };
+    spans
 }
 
 fn get_key() -> char {
     loop {
-        match event::read().unwrap() {
-            event::Event::Key(event) => match event.code {
-                event::KeyCode::Char(c) => return c,
-                _ => (),
-            },
-            _ => (),
+        if let event::Event::Key(event) = event::read().unwrap() {
+            if let event::KeyCode::Char(c) = event.code {
+                return c;
+            }
         }
     }
 }
@@ -160,11 +184,20 @@ fn get_line(terminal: &mut Term) -> Option<String> {
     for byte in stdin().bytes() {
         let next_char = byte.unwrap() as char;
         match next_char {
+            '\x08' | '\x7f' => {
+                if line.len() > 0 {
+                    line = line[0..line.len() - 1].to_string();
+                    stdout().execute(MoveLeft(1)).unwrap();
+                    print!(" ");
+                    stdout().execute(MoveLeft(1)).unwrap();
+                };
+            }
             '\x1B' => {
                 abort = true;
                 break;
             }
-            '\n' => {
+            // TODO handle newlines in tweet/reply
+            '\n' | '\r' => {
                 abort = false;
                 break;
             }
@@ -184,7 +217,7 @@ fn get_line(terminal: &mut Term) -> Option<String> {
     }
 }
 
-fn prepare_exit() {
+fn exit_gracefully() {
     stdout()
         .execute(terminal::Clear(terminal::ClearType::All))
         .unwrap()
@@ -194,17 +227,11 @@ fn prepare_exit() {
 }
 
 async fn run_action(tweet: &Tweet, key: char, token: &Token, terminal: &mut Term) -> String {
-    if key == 'x' {
-        prepare_exit();
-        exit(0);
-    };
+    draw(tweet_view(tweet), vec![Spans::from("Working...")], terminal);
+
     if key == 'l' {
-        match send_like(tweet, token).await {
-            Ok(()) => "Done!".to_string(),
-            Err(e) => err_view(e),
-        }
+        result_view(send_like(tweet, token).await)
     } else if key == 'r' {
-        // terminal::disable_raw_mode().unwrap();
         draw(
             tweet_view(tweet),
             vec![Spans::from("Your reply:")],
@@ -213,22 +240,32 @@ async fn run_action(tweet: &Tweet, key: char, token: &Token, terminal: &mut Term
         let height = terminal::size().unwrap().1;
         stdout().execute(MoveTo(13, height - 2)).unwrap();
         let reply = get_line(terminal);
-        // terminal::enable_raw_mode().unwrap();
 
         if reply.is_some() {
-            match send_reply(tweet, reply.unwrap(), token).await {
-                Ok(()) => "Done!".to_string(),
-                Err(e) => err_view(e),
-            }
+            result_view(send_reply(tweet, reply.unwrap(), token).await)
         } else {
-            "Aborted.".to_string()
+            "Nevermind.".to_string()
+        }
+    } else if key == 't' {
+        draw(
+            tweet_view(tweet),
+            vec![Spans::from("Your tweet:")],
+            terminal,
+        );
+        let height = terminal::size().unwrap().1;
+        stdout().execute(MoveTo(13, height - 2)).unwrap();
+        let text = get_line(terminal);
+        if text.is_some() {
+            result_view(send_tweet(text.unwrap(), token).await)
+        } else {
+            "Nevermind.".to_string()
         }
     } else {
-        "You can't do that".to_string()
+        "That doesn't do anything.".to_string()
     }
 }
 
-fn draw(text_0: Vec<Spans>, text_1: Vec<Spans>, terminal: &mut Term) {
+fn draw(main_text: Vec<Spans>, bottom_text: Vec<Spans>, terminal: &mut Term) {
     terminal
         .draw(|f| {
             let layout = Layout::default()
@@ -245,17 +282,59 @@ fn draw(text_0: Vec<Spans>, text_1: Vec<Spans>, terminal: &mut Term) {
                 layout[0],
             );
             f.render_widget(
-                Paragraph::new(text_0).block(Block::default().borders(Borders::ALL)),
+                Paragraph::new(main_text)
+                    .wrap(Wrap { trim: false })
+                    .block(Block::default().title("Mentions").borders(Borders::ALL)),
                 layout[1],
             );
             f.render_widget(
-                Paragraph::new(text_1).block(Block::default().borders(Borders::ALL)),
+                Paragraph::new(bottom_text).block(Block::default().borders(Borders::ALL)),
                 layout[2],
             );
         })
         .unwrap();
 }
 
+async fn render_tweets(mut tweets: Vec<Tweet>, terminal: &mut Term, token: &Token) {
+    let length = tweets.len();
+    let mut i = 0;
+    while i < length {
+        let tweet = &tweets[i];
+        draw(tweet_view(tweet), vec![Spans::from("")], terminal);
+        loop {
+            let key = get_key();
+            let message = match key {
+                'n' => {
+                    i += 1;
+                    break;
+                }
+                'p' => {
+                    if i == 0 {
+                        exit_gracefully();
+                        return;
+                    }
+                    i -= 1;
+                    break;
+                }
+                'P' => match tweet.in_reply_to_status_id {
+                    Some(id) => {
+                        draw(tweet_view(tweet), vec![Spans::from("Working...")], terminal);
+                        tweets.insert(i + 1, show(id, token).await.unwrap().response);
+                        i += 1;
+                        break;
+                    }
+                    None => "This tweet has no parent.".to_string(),
+                },
+                'x' => {
+                    return;
+                }
+                c => run_action(tweet, c, token, terminal).await,
+            };
+            draw(tweet_view(tweet), vec![Spans::from(message)], terminal);
+        }
+    }
+}
+
 #[tokio::main]
 async fn main() {
     println!("Authenticating...");
@@ -271,30 +350,8 @@ async fn main() {
         &mut terminal,
     );
     let tweets = get_mentions(&token).await;
-
     terminal::enable_raw_mode().unwrap();
-    for tweet in tweets {
-        draw(tweet_view(&tweet), vec![Spans::from("")], &mut terminal);
-        loop {
-            let key = get_key();
-            let message = match key {
-                'n' => break,
-                c => {
-                    draw(
-                        tweet_view(&tweet),
-                        vec![Spans::from("Working...")],
-                        &mut terminal,
-                    );
-                    run_action(&tweet, c, &token, &mut terminal).await
-                }
-            };
-            draw(
-                tweet_view(&tweet),
-                vec![Spans::from(message)],
-                &mut terminal,
-            );
-        }
-    }
+    render_tweets(tweets, &mut terminal, &token).await;
 
-    prepare_exit();
+    exit_gracefully();
 }
